@@ -23,17 +23,17 @@ import dash_html_components as html
 from dash.dependencies import Input, Output, State
 import dash_table
 import plotly.express as px
+import plotly.graph_objects as go
 import webbrowser
 
 from macrodemos.common_components import app_parameter, app_parameter_row, editable_cell_format
 
-
 import numpy as np
 from numpy.polynomial import Polynomial as P
 import pandas as pd
 from scipy import signal
 
-from statsmodels.tsa.arima_model import ARMA as tsaARMA # PARA ESTIMAR MODELO
+from statsmodels.tsa.arima.model import ARIMA as tsaARMA # PARA ESTIMAR MODELO
 from statsmodels.tsa.arima_process import ArmaProcess # MODELO TEORICO
 from statsmodels.tsa.stattools import acf, pacf # autocorrelation and partial autocorrelation
 
@@ -101,6 +101,32 @@ def validate_lists_of_values(value: str):
     return [float(x) for x in value.split(',')] if value else None
 
 
+def add_polar_scatter(fig, roots, name, ms=16):
+    """
+    Adds a polar scatter to existing figure
+
+    To help making the plots for the roots of the ARIMA polynomial
+
+    :param fig: existing plotly.Figure
+    :param roots: roots of the characteristic polynomial
+    :param name: name for the series (to use in legend)
+    :param ms: marker size
+    :return:
+    float, maximum absolute value of roots.
+    Updates fig in place
+    """
+    radium = np.abs(roots)
+    angles = np.angle(roots, True)
+    fig.add_trace(go.Scatterpolar(
+        r=radium,
+        theta=angles,
+        name = name,
+        mode = 'markers',
+        marker_size=ms
+    ))
+    return max(radium)
+
+
 #=======================================================================================================================
 #
 #  ARMA CLASS
@@ -212,8 +238,10 @@ class ARMA(ArmaProcess):
 
     def estimate(self, p, q):
         try:
-            res = tsaARMA(self.simulated_data.values, order=[p, q]).fit()
-            self.estimates = {'c': res.params[0], 'phi': res.arparams, 'theta': res.maparams}
+            res = tsaARMA(self.simulated_data.values, order=[p, 0, q]).fit()
+            self.estimates = {'c': res.params[0]}
+            self.estimates['phi'] = res.arparams if p else None
+            self.estimates['theta'] = res.maparams if q else None
             self.estimates['repr'] = str(ARMA(self.estimates['c'], self.estimates['phi'], self.estimates['theta']))
             self.estimates['fitted'] = pd.Series(res.fittedvalues)
             self.estimates['arroots'] = np.array([1/x for x in res.arroots])
@@ -230,8 +258,6 @@ class ARMA(ArmaProcess):
                 data=result.to_dict('records'),
                 columns=[{'name': i, 'id': i} for i in result.columns])
             self.estimated_table = new_estimate.data
-            print(self.estimated_table)
-
         except:
             self.estimated = False
             self.estimates['repr'] = 'The model could not be estimated'
@@ -240,49 +266,70 @@ class ARMA(ArmaProcess):
     def print2moments(self):
         return f'$E(y_t) = {self.mean:g}, V(y_t) = {self.variance:g}$' if self.isstationary else 'This process is not stationay!'
 
-
     def __str__(self):
         c = f'{self.c:g}' if self.c else ''
         if self.p:
-            ar = ' '.join([f'{phi:+g}y_{{t-{k+1}}}' for k, phi in enumerate(self.arcoefs) if phi])
+            ar = ' '.join([f'{phi:+g}y_{{t-{k + 1}}}' for k, phi in enumerate(self.arcoefs) if phi])
         else:
             ar = ''
         if self.q:
-            ma = ' '.join([f'{theta:+g}\epsilon_{{t-{k+1}}}' for k, theta in enumerate(self.macoefs) if theta])
+            ma = ' '.join([f'{theta:+g}\epsilon_{{t-{k + 1}}}' for k, theta in enumerate(self.macoefs) if theta])
         else:
             ma = ''
 
-        return r'$y_t = ' + c + ar + ' + \epsilon_t' + ma + '$'
+        rhs = c + ar + ' + \epsilon_t' + ma
+        rhs = rhs[1:] if rhs[0] == '+' else rhs  # drop leading + sign
+
+        return r'$y_t = ' + rhs + '$'
 
     def periodogram(self, frecuencias):
         return abs(signal.freqz(self.ma, self.ar, worN=frecuencias)[1]) ** 2 / (2 * np.pi)
 
     def plot_spectral(self):
-        results = {'layout': {'title': 'Spectral Density'}}
-        if self.isstationary:
-            results['data'] = [{'x': omega, 'y': self.periodogram(omega), 'type': 'line', 'name': 'actual', 'fill': 'tozeroy'}]
-        else:
-            results['data'] = [{'x': omega, 'y': np.zeros_like(omega), 'type': 'line', 'name': 'not defined'}]
+        fig = go.Figure()
+        fig.add_trace(go.Scatter(
+            x=omega,
+            y=self.periodogram(omega) if self.isstationary else np.zeros_like(omega),
+            name='actual' if self.isstationary else 'not defined',
+            fill='tozeroy')
+        )
+
         if self.show_estimates:
             yy = self.simulated_data - self.simulated_data.mean()
             v = (yy ** 2).mean()  # estimated variance
             T = yy.size
             r = int(np.sqrt(T))
-            gamma = acf(yy, unbiased=False, nlags=r, fft=False)
+            gamma = acf(yy, adjusted=False, nlags=r, fft=False)
             k = np.arange(r + 1)
-            g = 1 - k / r # Bartlett window
+            g = 1 - k / r  # Bartlett window
             sw = ((np.cos(np.outer(omega, k)) * (g * gamma)).sum(axis=1) * 2 - 1)
             sw *= v / (2 * np.pi)  # rescale
-            results['data'].append({'x': omega, 'y': sw, 'type': 'line', 'name': 'estimated'})
 
-        return results
+            fig.add_trace(go.Scatter(
+                x=omega,
+                y=sw,
+                mode='lines',
+                name='estimated'
+            )
+            )
+
+        fig.update_layout(
+            title='Spectral Density',
+            template='simple_white',
+            xaxis=dict(
+                tickmode='array',
+                tickvals=np.linspace(0, np.pi, 5),
+                ticktext=['0', r'$\pi/4$', r'$\pi/2$', r'$3\pi/4$', r'$\pi$']
+            )
+        )
+        return fig
 
     def plot_correlogram(self, lags):
         lags += 1
 
         df = pd.DataFrame({'actual': self.acf(lags=lags)} if self.isstationary else {'not defined':np.zeros(lags)})
         if self.show_estimates:
-            self.estimates['acf'] = acf(self.simulated_data, unbiased=True, nlags=lags-1, fft=False)
+            self.estimates['acf'] = acf(self.simulated_data, adjusted=True, nlags=lags-1, fft=False)
             df['estimated'] = self.estimates['acf']
 
         fig = px.bar(df,
@@ -350,72 +397,54 @@ class ARMA(ArmaProcess):
         return fig
 
     def plot_ar_roots(self):
-        results = dict()
-
+        fig = go.Figure()
         if self.p:
-            roots = 1/self.arroots #statsmodels gives roots of the lag polinomial, not the characteristic function
-            radium = np.abs(roots)
-            angles = np.angle(roots, True)
-            results['data'] = [{'r': radium, 'theta': angles, 'type': 'scatterpolar','mode': 'markers', 'marker': {'size': 16}, 'name': 'actual'}]
+            rmax0 = add_polar_scatter(fig, 1 / self.arroots,
+                                      'actual')  # statsmodels gives roots of the lag polinomial, not the characteristic function
         else:
-            radium = np.ones(2)
-            results['data'] = [{'r': [0], 'theta': [0], 'type': 'scatterpolar','mode': 'markers','marker': {'size': 1}, 'name': 'Actual'}]
+            rmax0 = add_polar_scatter(fig, [0], 'actual', ms=0)
 
         if self.show_estimates and self.estimated:
             roots = self.estimates['arroots']
             if len(roots):
-                radium2 = np.abs(roots)
-                angles2 = np.angle(roots, True)
-
-                results['data'].append(
-                    {'r': radium2, 'theta': angles2, 'type': 'scatterpolar', 'mode': 'markers', 'marker': {'size': 16},
-                     'name': 'estimated'})
+                rmax1 = add_polar_scatter(fig, roots, 'estimated')
             else:
-                radium2 = np.ones(2)
+                rmax1 = 1
         else:
-            radium2 = np.ones(2)
+            rmax1 = 1
 
-        maxr = max([1, radium.max(), radium2.max()])
-        results['layout'] = {'title': 'AR inverse roots',
-                             'polar': {'angularaxis': {'thetaunit': 'radians', 'dtick': np.pi / 4},
-                                       'radialaxis': {'tickvals': [0.0, 1.0], 'range': [0, maxr]}}
-                             }
-
-        return results
+        maxr = max([1, rmax0, rmax1])
+        fig.update_layout(
+            title='AR inverse roots',
+            polar={'angularaxis': {'thetaunit': 'radians', 'dtick': np.pi / 4},
+                   'radialaxis': {'tickvals': [0.0, 1.0], 'range': [0, maxr]}}
+        )
+        return fig
 
     def plot_ma_roots(self):
-        results = dict()
-
+        fig = go.Figure()
         if self.q:
-            roots = 1/self.maroots #statsmodels gives roots of the lag polinomial, not the characteristic function
-            radium = np.abs(roots)
-            angles = np.angle(roots, True)
-            results['data'] = [{'r': radium, 'theta': angles, 'type': 'scatterpolar','mode': 'markers', 'marker': {'size': 16}, 'name': 'actual'}]
+            rmax0 = add_polar_scatter(fig, 1 / self.maroots,
+                                      'actual')  # statsmodels gives roots of the lag polinomial, not the characteristic function
         else:
-            radium = np.ones(2)
-            results['data'] = [{'r': [0], 'theta': [0], 'type': 'scatterpolar','mode': 'markers','marker': {'size': 1}, 'name': 'Actual'}]
+            rmax0 = add_polar_scatter(fig, [0], 'actual', ms=0)
 
         if self.show_estimates and self.estimated:
             roots = self.estimates['maroots']
             if len(roots):
-                radium2 = np.abs(roots)
-                angles2 = np.angle(roots, True)
-
-                results['data'].append(
-                    {'r': radium2, 'theta': angles2, 'type': 'scatterpolar', 'mode': 'markers', 'marker': {'size': 16},
-                     'name': 'estimated'})
+                rmax1 = add_polar_scatter(fig, roots, 'estimated')
             else:
-                radium2 = np.ones(2)
+                rmax1 = 1
         else:
-            radium2 = np.ones(2)
+            rmax1 = 1
 
-        maxr = max([1, radium.max(), radium2.max()])
-        results['layout'] = {'title': 'MA inverse roots',
-                             'polar': {'angularaxis': {'thetaunit': 'radians', 'dtick': np.pi / 4},
-                                       'radialaxis': {'tickvals': [0.0, 1.0], 'range': [0, maxr]}}
-                             }
-
-        return results
+        maxr = max([1, rmax0, rmax1])
+        fig.update_layout(
+            title='MA inverse roots',
+            polar={'angularaxis': {'thetaunit': 'radians', 'dtick': np.pi / 4},
+                   'radialaxis': {'tickvals': [0.0, 1.0], 'range': [0, maxr]}}
+        )
+        return fig
 
     def plot_irf(self, horizon):
 
@@ -607,7 +636,7 @@ def ARMA_demo(colab=False):
         app.run_server(mode='external')
     else:
         webbrowser.open('http://127.0.0.1:8050/')
-        app.run_server(debug=False)
+        app.run_server(debug=True)
 
 
 if __name__ == '__main__':
