@@ -269,7 +269,7 @@ def update_ARMA_parameters(n_clicks,T, α, δ, n, A, s, α1, δ1, n1, A1, s1):
     df['% change'] = 100 * (df['Alternative'] / df['Baseline'] - 1)
     df = df.round(3).loc[modelo.endogenous]
     df.index.name = 'Variable'
-    return *[modelo.plot_field(ser) for ser in modelo.endogenous], df.reset_index().to_dict('records')
+    return [modelo.plot_field(ser) for ser in modelo.endogenous].append(df.reset_index().to_dict('records'))
 
 
 
