@@ -437,8 +437,8 @@ app.layout = html.Div(children=[
 """==========================================================================================================
 This block defines how the app responds to all inputs and states
 """
-#DATA = {'original': get_data('Costa Rica', 'Seasonally Adjusted')}
-DATA = {'original': pd.read_pickle('./data/CRI-initial-data.pickle')}
+initial_data = os.path.join(os.path.dirname(os.path.abspath(__file__)), 'data', 'CRI-initial-data.csv')
+DATA = {'original': pd.read_csv(initial_data, parse_dates=True, index_col=0)}
 
 
 @app.callback(
